from formaloo import constants, client, helper


class Tag(helper.RequestHandler):

    def __init__(self, title=None, description=None, slug=None):
        self.client = client.client
        self.title = title
        self.description = description
        self.slug = slug
        self.actions = {
            "get_list": {
                "url": constants.V_1_0_TAG_LIST_CREATE_ENDPOINT,
                "has_url_params": False,
                "body": None,
                "method": self.client.get
            },
            "create": {
                "url": constants.V_1_0_TAG_LIST_CREATE_ENDPOINT,
                "has_url_params": False,
                "body": self.get_body(),
                "method": self.client.post
            },
            "get": {
                "url": constants.V_1_0_TAG_ITEM_ENDPOINT,
                "has_url_params": True,
                "body": None,
                "method": self.client.get
            }
        }

    def get_body(self):
        body = {
            'title': self.title
        }

        if self.description:
            body['description'] = self.description

        if self.slug:
            body['slug'] = self.slug

        return body

    @staticmethod
    def get_list_body(tags):
        body = []

        for tag in tags:
            body.append(
                tag.get_body()
            )

        return body



from formaloo import constants, helper


class FormDisplay(helper.RequestHandler):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.actions = {
            "get_by_address": {
                "url": constants.V_1_0_FORM_DISPLAY_ADDRESS_ENDPOINT,
                "has_url_params": True,
                "body": None,
                "method": self.client.get
            },
            "get_by_slug": {
                "url": constants.V_1_0_FORM_DISPLAY_SLUG_ENDPOINT,
                "has_url_params": True,
                "body": None,
                "method": self.client.get
            },
            "submit": {
                "url": constants.V_1_0_FORM_DISPLAY_SUBMIT_ENDPOINT,
                "has_url_params": True,
                "body": self.get_body(),
                "method": self.client.post
            }
        }



from formaloo import constants, client, helper


class CustomerBatch(helper.RequestHandler):

    def __init__(self, customers=None):
        if not customers:
            customers = []

        self.client = client.client
        self.customers = customers
        self.actions = {
            "create": {
                "url": constants.V_1_0_CUSTOMER_BATCH_ENDPOINT,
                "has_url_params": False,
                "body": self.get_body(),
                "method": self.client.post
            }
        }

    def add_customer(self, customer):
        self.customers.append(customer)

    def get_body(self):
        body = {
            'customers_data': []
        }

        for customer in self.customers:
            body['customers_data'].append(
                customer.get_body()
            )

        return body



from formaloo import constants, helper


class GamificationCalculationJob(helper.RequestHandler):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.actions = {
            "create": {
                "url": constants.V_1_0_CREATE_GAMIFICATION_CALCULATION_JOB,
                "has_url_params": False,
                "body": self.get_body(),
                "method": self.client.post
            }
        }



from formaloo import constants, helper


class FormTemplate(helper.RequestHandler):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.actions = {
            "get_list": {
                "url": constants.V_1_0_FORM_TEMPLATE_LIST_ENDPOINT,
                "has_url_params": False,
                "body": None,
                "method": self.client.get
            }
        }



from formaloo import constants, helper


class Form(helper.RequestHandler):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.actions = {
            "get_list": {
                "url": constants.V_1_0_FORM_LIST_CREATE_ENDPOINT,
                "has_url_params": False,
                "body": None,
                "method": self.client.get
            },
            "create": {
                "url": constants.V_1_0_FORM_LIST_CREATE_ENDPOINT,
                "has_url_params": False,
                "body": self.get_body(),
                "method": self.client.post
            },
            "get": {
                "url": constants.V_1_0_FORM_ITEM_ENDPOINT,
                "has_url_params": True,
                "body": None,
                "method": self.client.get
            },
            "get_stats": {
                "url": constants.V_1_0_FORM_LIST_CREATE_ENDPOINT,
                "has_url_params": True,
                "body": None,
                "method": self.client.get
            },
            "patch": {
                "url": constants.V_1_0_FORM_ITEM_ENDPOINT,
                "has_url_params": True,
                "body": self.get_body(),
                "method": self.client.patch
            },
            "delete": {
                "url": constants.V_1_0_FORM_ITEM_ENDPOINT,
                "has_url_params": True,
                "body": None,
                "method": self.client.delete
            }
        }



from .form import Form
from .form_display import FormDisplay
from .category import FormCategory
from .field import Field
from .payment_method import PaymentMethod
from .row_tag import RowTag
from .row_vote import RowVote
from .template import FormTemplate




from typing import List, Dict

from formaloo import client


class RequestHandler:

    """
    actions example:
    self.actions = {
        "get_list": {
            "url": constants.V_1_0_FORM_LIST_CREATE_ENDPOINT,
            "has_url_params": False,
            "body": None,
            "method": self.client.get
        },
        "create": {
            "url": constants.V_1_0_FORM_LIST_CREATE_ENDPOINT,
            "has_url_params": False,
            "body": self.get_body(),
            "method": self.client.post
        }
    }
    """

    def __init__(self, **kwargs):
        self.client = client.Client()
        self.data = kwargs
        self.actions = {}

    def get_body(self):
        body = {}
        body.update(self.data)
        return body

    def validate(self, action, url_params):
        if action.get('has_url_params') and not url_params:
            raise ValueError(
                "`url_params` parameter is required."
            )

    @property
    def actions_list(self):
        return list(self.actions.keys())

    def __getattr__(self, attr):
        if attr not in self.actions_list:
            raise AttributeError(
                "{} has no action {}".format(
                    self.__class__.__name__,
                    attr
                )
            )
        self.received_action = attr
        return self._send_request

    def _send_request(
        self,
        url_params: List[str] = None,
        **kwargs
    ) -> Dict:
        """
        url_patrams: ["slug1", "slug2"]

        Example of using this method in projects:

        from formaloo.forms import Form
        form = Form()
        form.get_stats(
            url_params=['9fpTfgpR']
        )
        """

        action = self.actions.get(self.received_action)
        self.validate(action, url_params)

        url = action.get('url')
        request_data = {}

        if action['method'] in [
            self.client.get,
            self.client.delete
        ]:
            request_data['params'] = kwargs

        if action.get('body'):
            request_data['body'] = action.get('body')

        if action.get('has_url_params'):
            url = url.format(*url_params)

        response = action['method'](
            url,
            **request_data
        )

        return response.json()



from formaloo import constants, client, helper
from formaloo.tags import Tag


class Activity(helper.RequestHandler):

    def __init__(
        self, customer_data=None,
        activity_data=None, tags=None,
        **kwargs
    ):
        if not activity_data:
            activity_data = {}

        if not tags:
            tags = []

        if not customer_data:
            customer_data = {}

        self.client = client.client
        self.customer_data = customer_data
        self.activity_data = activity_data
        self.action = kwargs.get('action', None)
        self.activity_date = kwargs.get('activity_date', None)
        self.relations = kwargs.get('relations', None)
        self.monetary_value = kwargs.get('monetary_value', None)
        self.currency = kwargs.get('currency', None)
        self.tags = tags
        self.actions = {
            "get_list": {
                "url": constants.V_1_0_ACTIVITY_LIST_CREATE_ENDPOINT,
                "has_url_params": False,
                "body": None,
                "method": self.client.get
            },
            "get": {
                "url": constants.V_1_0_ACTIVITY_ITEM_ENDPOINT,
                "has_url_params": True,
                "body": None,
                "method": self.client.get
            },
            "create": {
                "url": constants.V_1_0_ACTIVITY_LIST_CREATE_ENDPOINT,
                "has_url_params": False,
                "body": self.get_body(),
                "method": self.client.post
            }
        }

    def get_body(self):
        tags_body = Tag.get_list_body(self.tags)

        body = {
            'action': self.action,
            'customer': self.customer_data,
            'activity_data': self.activity_data,
            'activity_date': self.activity_date,
            'relations': self.relations,
            'monetary_value': self.monetary_value,
            'currency': self.currency,
            'tags': tags_body
        }

        return body



from formaloo import constants, client, helper


class ActivityBatch(helper.RequestHandler):

    def __init__(self, activities=[]):
        self.client = client.client
        self.activities = activities
        self.actions = {
            "create": {
                "url": constants.V_1_0_ACTIVITY_BATCH_ENDPOINT,
                "has_url_params": False,
                "body": self.get_body(),
                "method": self.client.post
            }
        }

    def add_activity(self, activity):
        self.activities.append(activity)

    def get_body(self):
        body = {
            'activities_data': []
        }

        for activity in self.activities:
            body['activities_data'].append(
                activity.get_body()
            )

        return body
